@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -64,7 +64,7 @@ USA.
 		       (bindings '()))
 	      (if (not (pair? elements))
 		  (reverse!
-		   (cons `(define ,(symbol-append '* name '*)
+		   (cons `(define ,(symbol '* name '*)
 			    '#(,@(reverse! bindings)))
 			 code))
 		  (let* ((next (car elements))
@@ -78,7 +78,7 @@ USA.
 				      (error "define-enumeration: Overlap"
 					     next)
 				      m)))))
-		    (let ((name (symbol-append name '/ suffix)))
+		    (let ((name (symbol name '/ suffix)))
 		      (loop (+ n 1)
 			    (cdr elements)
 			    (cons `(DEFINE-INTEGRABLE ,name ,n)
@@ -359,20 +359,13 @@ push-primitive-7			; name in string table
 	unspecific)))
 
 (define (stackify/c-quotify str)
-  (let* ((len (string-length str))
-	 (res (make-string len)))
-    (do ((i 0 (1+ i)))
-	((>= i len) res)
-      (let ((c (string-ref str i)))
-	(case c
-	  ((#\*)
-	   (string-set! res i #\S))
-	  ((#\- #\/)
-	   (string-set! res i #\_))
-	  ((#\+)
-	   (string-set! res i #\P))
-	  (else
-	   (string-set! res i c)))))))
+  (string-map (lambda (c)
+		(case c
+		  ((#\*) #\S)
+		  ((#\- #\/) #\_)
+		  ((#\+) #\P)
+		  (else c)))
+	      str))
 
 (define (stackify/dump-c-enums output)
   (with-output-to-file output
@@ -402,7 +395,7 @@ push-primitive-7			; name in string table
 	    (for-each
 	     write-string
 	     (list "\t"
-		   (stackify/C-quotify (symbol-name (car binding)))
+		   (stackify/C-quotify (symbol->string (car binding)))
 		   " = 0"
 		   (if (zero? value)
 		       ""
