@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -83,23 +83,20 @@ USA.
 	      "comatch"
 	      "display"
 	      "key-w32"
-	      "key-x11"
 	      "macros"
 	      "make"
 	      "nntp"
 	      "nvector"
-	      "os2term"
 	      "paths"
 	      "rcsparse"
-	      "rename"
 	      "rfc822"
 	      "ring"
+	      "string"
 	      "strpad"
 	      "strtab"
 	      "termcap"
 	      "utils"
 	      "win32"
-	      "winren"
 	      "xform"
 	      "xterm"))
   (sf-edwin "tterm" "termcap")
@@ -139,7 +136,6 @@ USA.
 		"debuge"
 		"diff"
 		"dired"
-		"diros2"
 		"dirunx"
 		"dirw32"
 		"docstr"
@@ -184,8 +180,6 @@ USA.
 		"notify"
 		"outline"
 		"occur"
-		"os2"
-		"os2com"
 		"paredit"
 		"pasmod"
 		"print"
@@ -234,9 +228,8 @@ USA.
 		"webster"
 		"wincom"
 		"winout"
-		"xcom"
 		"win32com"
-		"xmodef")))
+		"world-monitor")))
   (for-each sf-class
 	    '("comwin"
 	      "modwin"
