@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: screen.scm,v 1.124 2007/01/05 21:19:24 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -134,6 +133,9 @@ USA.
   (editor-frame-select-window! (screen-root-window screen) window)
   (screen-modeline-event! screen window 'SELECT-WINDOW))
 
+(define-integrable (screen-cursor-window screen)
+  (editor-frame-cursor-window (screen-root-window screen)))
+
 (define-integrable (screen-select-cursor! screen window)
   (editor-frame-select-cursor! (screen-root-window screen) window))
 
