@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -29,7 +30,7 @@ USA.
 
 (define (make-console-screen)
   (let ((description (console-termcap-description)))
-    (cond ((not (output-port/baud-rate console-i/o-port))
+    (cond ((not (output-port/baud-rate (console-i/o-port)))
 	   (error "standard output not a terminal"))
 	  ((not description)
 	   (error "terminal type not set"))
@@ -41,9 +42,9 @@ USA.
 	  ((not (no-undesirable-characteristics? description))
 	   (error "terminal type has undesirable characteristics"
 		  (terminal-type-name description))))
-    (let ((baud-rate (output-port/baud-rate console-i/o-port))
-	  (x-size (output-port/x-size console-i/o-port))
-	  (y-size (output-port/y-size console-i/o-port)))
+    (let ((baud-rate (output-port/baud-rate (console-i/o-port)))
+	  (x-size (output-port/x-size (console-i/o-port)))
+	  (y-size (output-port/y-size (console-i/o-port))))
       (make-screen (with-values
 		       (lambda ()
 			 (compute-scrolling-costs description
@@ -96,7 +97,7 @@ USA.
 	 (terminal-output-baud-rate channel))))
 
 (define (output-port/buffered-bytes port)
-  (let ((operation (port/operation port 'BUFFERED-OUTPUT-BYTES)))
+  (let ((operation (textual-port-operation port 'buffered-output-bytes)))
     (if operation
 	(operation port)
 	0)))
@@ -108,11 +109,11 @@ USA.
 	 (no-undesirable-characteristics? description))))
 
 (define (console-termcap-description)
-  (if (eq? console-description 'UNKNOWN)
+  (if (eq? console-description 'unknown)
       (set! console-description
 	    (let ((term (get-environment-variable "TERM")))
 	      (and term
-		   (or (and (output-port/baud-rate console-i/o-port)
+		   (or (and (output-port/baud-rate (console-i/o-port))
 			    (make-termcap-description term))
 		       term)))))
   console-description)
@@ -159,154 +160,198 @@ USA.
      )))
 
 (define (get-console-input-operations terminal-state)
-  (let ((channel (port/input-channel console-i/o-port))
-        (string  (make-string (* 3 input-buffer-size)))
+  ;; When the input is a prefix of the character sequence sent by some
+  ;; key, we are prepared to wait a little-while to see if the rest of
+  ;; the sequence arrives.
+
+  ;; These procedures read buffer-fuls of input octets and match the
+  ;; terminal's special key sequences against the buffer.  They wait a
+  ;; little-while for incomplete sequences, then yield the individual
+  ;; characters.
+  (let ((channel (port/input-channel (console-i/o-port)))
+        (buffer  (make-string (* 3 input-buffer-size)))
         (start   0)
         (end     0)
-        (incomplete-pending #F)
-	(timeout-interval 1000)		; 1s. Should be f(baud rate) etc
-        (len     0))                    ; length of event in input characters
-    ;; When the input is a prefix of the character sequence sent by some key
-    ;; we are prepared to wait a little while to see if the rest of
-    ;; the sequence arrives.  INCOMPLETE-PENDING is either #F, the
-    ;; real time at which we timeout for waiting for the sequence to
-    ;; complete, or #T if a timeout occured.
+	(little-while 500)		; .5sec. Should be f(baud rate) etc
+
+	;; INCOMPLETE-PENDING is either #F, the real time at which we
+	;; should stop waiting for the sequence to complete, or #T if
+	;; we are no longer waiting.  It is set in parse-key when an
+	;; incomplete sequence is first matched, and is not cleared
+	;; until something (special-key or individual character) is
+	;; read (consumed, not peeked).  Thus many peeks and a
+	;; subsequent read do not EACH wait a little-while.
+        (incomplete-pending #F))
+
     (letrec
-        ((parse-key			; -> #F or a char? or a special-key?
-	  (lambda ()
+        ((match-key			; -> match: #F or char or (seq . name)
+	  (named-lambda (match-key)
 	    (and (fix:< start end)
 		 terminal-state
-		 (let ((n-chars  (fix:- end start)))
+		 (let ((n-chars (fix:- end start)))
 		   (let find
 		       ((key-pairs (terminal-state/key-table terminal-state))
-			(possible-pending? #F))
+			(possible-pending? #f))
 		     (if (null? key-pairs)
 			 (begin
 			   (if (number? incomplete-pending)
 			       (if (or (not possible-pending?)
 				       (> (real-time-clock)
 					  incomplete-pending))
-				   (set! incomplete-pending #T)))
+				   (set! incomplete-pending #t)))
 			   (if (number? incomplete-pending)
-			       #F
-			       (begin
-				 (set! len 1)
-				 ;; We must explicitly map the 8th bit
-				 ;; of an incoming character to the
-				 ;; meta bit.
-				 (let ((code (vector-8b-ref string start)))
-				   (if (fix:< code #x80)
-				       (make-char code 0)
-				       (make-char (fix:and code #x7F)
-						  char-bit:meta))))))
+			       #f
+			       (vector-8b-ref buffer start)))
 			 (let* ((key-seq  (caar key-pairs))
 				(n-seq    (string-length key-seq)))
 			   (cond ((and (fix:<= n-seq n-chars)
-				       (substring=? string start
+				       (substring=? buffer start
 						    (fix:+ start n-seq)
 						    key-seq 0 n-seq))
-				  (set! len n-seq)
-				  (cdar key-pairs))
+				  (car key-pairs))
 				 ((and (fix:> n-seq n-chars)
-				       (substring=? string start
+				       (substring=? buffer start
 						    (fix:+ start n-chars)
 						    key-seq 0 n-chars))
 				  (if (not incomplete-pending)
 				      (set! incomplete-pending
 					    (+ (real-time-clock)
-					       timeout-interval)))
+					       little-while)))
 				  (find (cdr key-pairs) #T))
 				 (else
 				  (find (cdr key-pairs)
 					possible-pending?))))))))))
-	 (read-more?			; -> #F or #T is some chars were read
-	  (lambda (block?)
-	    (if block?
-		(channel-blocking channel)
-		(channel-nonblocking channel))
-	    (let ((n (channel-read channel string end input-buffer-size)))
-	      (cond ((not n)  #F)
+	 (read-more?			; -> #F or #T if some octets were read
+	  (named-lambda (read-more?)
+	    (let ((n (%channel-read channel buffer end input-buffer-size)))
+	      (cond ((not n)  #f)
+		    ((eq? n #t) #f)
 		    ((fix:> n 0)
 		     (set! end (fix:+ end n))
-		     #T)
+		     #t)
 		    ((fix:= n 0)
 		     ;;(error "Reached EOF in keyboard input.")
-		     #F)
-		    (else
-		     (error "Illegal return value:" n))))))
-	 (read-char
-	  (lambda (block?)
-	    (if (read-more? block?)
-		(parse-key)
-		#F)))
-	 (read-event
-	  (lambda (block?)
-	    (or (read-char #f)
-		(let loop ()
-		  (cond (inferior-thread-changes? event:interrupt)
-			((process-output-available?) event:process-output)
-			((not have-select?)
-			 (and block? (read-event block?)))
-			(else
-			 (case (test-for-io-on-channel channel 'READ block?)
-			   ((#F) #f)
-			   ((PROCESS-STATUS-CHANGE) event:process-status)
-			   ((INTERRUPT) (loop))
-			   (else (read-event block?)))))))))
-	 (guarantee-result
-	  (lambda ()
-	    (let ((event (read-event #t)))
-	      (cond ((char? event) event)
-		    ((special-key? event) event)
-		    ((process-change-event event)
-		     => (lambda (flag)
-			  (make-input-event
-			   (if (eq? flag 'FORCE-RETURN) 'RETURN 'UPDATE)
-			   update-screens! #f)))
-		    (else (guarantee-result))))))
-	 (consume!
-	  (lambda (bytes)
-	    (set! start (fix:+ start bytes))
-	    (cond ((fix:>= start end)	; all consumed
-		   (set! end 0)
-		   (set! start 0))
+		     #f)))))
+	 (match-event	; -> #F or match (char or pair) or input event
+	  (named-lambda (match-event block?)
+	    (let loop ()
+	      (or (begin
+		    (read-more?)
+		    (match-key))
+		  ;; Poll event sources and block.
+		  (begin
+		    (cond (inferior-thread-changes?
+			   (or (->update-event (accept-thread-output))
+			       (loop)))
+			  ((process-output-available?)
+			   (or (->update-event (accept-process-output))
+			       (loop)))
+			  ((process-status-changes?)
+			   (or (->update-event (handle-process-status-changes))
+			       (loop)))
+			  ((not have-select?)
+			   (and block? (loop)))
+			  (incomplete-pending
+			   ;; Must busy-wait.
+			   (loop))
+			  (block?
+			   (block-for-event)
+			   (loop))
+			  (else
+			   #f)))))))
+	 (->update-event
+	  (named-lambda (->update-event redisplay?)
+	    (and redisplay?
+		 (make-input-event
+		  (if (eq? redisplay? 'force-return) 'return 'update)
+		  update-screens! #f))))
+	 (consume-match!
+	  (named-lambda (consume-match! match)
+	    (cond ((fixnum? match)
+		   (set! start (fix:1+ start)))
+		  ((input-event? match)
+		   unspecific)
+		  ((pair? match)
+		   (set! start (fix:+ start (string-length (car match)))))
+		  (else (error "Inedible match:" match)))
+	    (if (fix:< end start)
+		(error "Overconsumption:" buffer start end match))
+	    (cond ((fix:= start end)	; all consumed
+		   (if (not (fix:zero? start))
+		       (set! start 0))
+		   (if (not (fix:zero? end))
+		       (set! end 0)))
 		  ((fix:>= start input-buffer-size)
-		   (substring-move-left! string start end string 0)
+		   (substring-move-left! buffer start end buffer 0)
 		   (set! end (fix:- end start))
 		   (set! start 0)))
-	    (set! incomplete-pending #F)
-	    unspecific)))
+	    (set! incomplete-pending #f)))
+	 (->event
+	  (named-lambda (->event match)
+	    (cond ((eq? match #f)
+		   #F)
+		  ((fixnum? match)
+		   ;; Assume the eighth bit is a meta bit.
+		   (if (fix:< match #x80)
+		       (make-char match 0)
+		       (make-char (fix:and match #x7F) char-bit:meta)))
+		  ((input-event? match)
+		   match)
+		  ((pair? match)
+		   (cdr match))
+		  (else (error "Bogus input match:" match)))))
+	 (block-for-event
+	  (named-lambda (block-for-event)
+	    (let ((input-available? #f)
+		  (output-available? #f)
+		  (registrations))
+	      (dynamic-wind
+	       (lambda ()
+		 (let ((thread (current-thread)))
+		   (set! registrations
+			 (cons
+			  (register-io-thread-event
+			   (channel-descriptor-for-select channel) 'read
+			   thread (lambda (mode)
+				    mode
+				    (set! input-available? #t)))
+			  (register-process-output-events
+			   thread (lambda (mode)
+				    mode
+				    (set! output-available? #t)))))))
+	       (lambda ()
+		 (with-thread-events-blocked
+		  (lambda ()
+		    (if (and (not input-available?)
+			     (not output-available?)
+			     (not (process-status-changes?))
+			     (not inferior-thread-changes?))
+			(suspend-current-thread))))
+		 unspecific)
+	       (lambda ()
+		 (for-each deregister-io-thread-event registrations)))))))
       (values
-       (lambda ()			;halt-update?
+       (named-lambda (halt-update?)
 	 (or (fix:< start end)
-	     (read-char #f)))
-       (lambda ()			;peek-no-hang
-	 (or (parse-key)
-	     (let ((event (read-event #f)))
-	       (if (fix:fixnum? event)
-		   (begin
-		     (process-change-event event)
-		     #f)
-		   event))))
-       (lambda ()			;peek
-	 (or (parse-key)
-	     (guarantee-result)))
-       (lambda ()			;read
-	 (let ((event (or (parse-key) (guarantee-result))))
-	   (consume! len)
-	   event))))))
-
+	     (read-more?)))
+       (named-lambda (peek-no-hang timeout)
+	 (keyboard-peek-busy-no-hang
+	  (lambda ()
+	    (let ((event (->event (match-event #f))))
+	      (if (input-event? event)
+		  (begin
+		    (apply-input-event event)
+		    #f)
+		  event)))
+	  timeout))
+       (named-lambda (peek)
+	 (->event (match-event #t)))
+       (named-lambda (read)
+	 (let ((match (match-event #t)))
+	   (consume-match! match)
+	   (->event match)))))))
 
 (define-integrable input-buffer-size 16)
-(define-integrable event:process-output -2)
-(define-integrable event:process-status -3)
-(define-integrable event:interrupt -4)
-
-(define (process-change-event event)
-  (cond ((fix:= event event:process-output) (accept-process-output))
-	((fix:= event event:process-status) (handle-process-status-changes))
-	((fix:= event event:interrupt) (accept-thread-output))
-	(else (error "Illegal change event:" event))))
 
 (define (signal-interrupt!)
   (signal-thread-event editor-thread
@@ -336,7 +381,7 @@ USA.
 
 (define (initialize-package!)
   (set! console-display-type
-	(make-display-type 'CONSOLE
+	(make-display-type 'console
 			   false
 			   console-available?
 			   make-console-screen
@@ -346,16 +391,17 @@ USA.
 			   with-console-grabbed
 			   with-console-interrupts-enabled
 			   with-console-interrupts-disabled))
-  (set! console-description 'UNKNOWN)
+  (set! console-description 'unknown)
   unspecific)
 
 (define (with-console-grabbed receiver)
   (bind-console-state false
     (lambda (get-outside-state)
       (terminal-operation terminal-raw-input
-			  (port/input-channel console-i/o-port))
+			  (port/input-channel (console-i/o-port)))
+      (channel-nonblocking (port/input-channel (console-i/o-port)))
       (terminal-operation terminal-raw-output
-			  (port/output-channel console-i/o-port))
+			  (port/output-channel (console-i/o-port)))
       (tty-set-interrupt-enables 2)
       (receiver
        (lambda (thunk)
@@ -378,14 +424,14 @@ USA.
 		    (set-console-state! outside-state)))))
 
 (define (console-state)
-  (vector (channel-state (port/input-channel console-i/o-port))
-	  (channel-state (port/output-channel console-i/o-port))
+  (vector (channel-state (port/input-channel (console-i/o-port)))
+	  (channel-state (port/output-channel (console-i/o-port)))
 	  (tty-get-interrupt-enables)))
 
 (define (set-console-state! state)
-  (set-channel-state! (port/input-channel console-i/o-port)
+  (set-channel-state! (port/input-channel (console-i/o-port))
 		      (vector-ref state 0))
-  (set-channel-state! (port/output-channel console-i/o-port)
+  (set-channel-state! (port/output-channel (console-i/o-port))
 		      (vector-ref state 1))
   (tty-set-interrupt-enables (vector-ref state 2)))
 
@@ -427,11 +473,11 @@ USA.
   (description false read-only true)
   (baud-rate-index false read-only true)
   (baud-rate false read-only true)
-  (insert-line-cost false read-only true)
-  (insert-line-next-cost false read-only true)
-  (delete-line-cost false read-only true)
-  (delete-line-next-cost false read-only true)
-  (scroll-region-cost false read-only true)
+  (insert-line-cost false)
+  (insert-line-next-cost false)
+  (delete-line-cost false)
+  (delete-line-next-cost false)
+  (scroll-region-cost false)
   (cursor-x false)
   (cursor-y false)
   (standout-mode? false)
@@ -444,22 +490,22 @@ USA.
   (sc-macro-transformer
    (lambda (form environment)
      (let ((name (cadr form)))
-       `(DEFINE-INTEGRABLE (,(symbol-append 'SCREEN- name) SCREEN)
-	  (,(close-syntax (symbol-append 'TERMINAL-STATE/ name)
+       `(define-integrable (,(symbol 'screen- name) screen)
+	  (,(close-syntax (symbol 'terminal-state/ name)
 			  environment)
-	   (SCREEN-STATE SCREEN)))))))
+	   (screen-state screen)))))))
 
 (define-syntax define-ts-modifier
   (sc-macro-transformer
    (lambda (form environment)
      (let ((name (cadr form)))
        (let ((param (make-synthetic-identifier name)))
-	 `(DEFINE-INTEGRABLE
-	    (,(symbol-append 'SET-SCREEN- name '!) SCREEN ,param)
+	 `(define-integrable
+	    (,(symbol 'set-screen- name '!) screen ,param)
 	    (,(close-syntax
-	       (symbol-append 'SET-TERMINAL-STATE/ name '!)
+	       (symbol 'set-terminal-state/ name '!)
 	       environment)
-	     (SCREEN-STATE SCREEN)
+	     (screen-state screen)
 	     ,param)))))))
 
 (define-ts-accessor description)
@@ -487,12 +533,13 @@ USA.
 
 (define (console-discard! screen)
   screen
-  (set! console-description 'UNKNOWN)
+  (set! console-description 'unknown)
   unspecific)
 
 (define (console-enter! screen)
   (add-event-receiver! event:console-resize resize-screen)
   (maybe-output screen (ts-enter-termcap-mode (screen-description screen)))
+  (maybe-output screen (ts-enter-keypad-mode (screen-description screen)))
   (set-screen-cursor-x! screen false)
   (set-screen-cursor-y! screen false))
 
@@ -502,25 +549,25 @@ USA.
     (move-cursor screen 0 (fix:-1+ (screen-y-size screen)))
     (exit-standout-mode screen)
     (exit-insert-mode screen)
+    (maybe-output screen (ts-exit-keypad-mode description))
     (maybe-output screen (ts-exit-termcap-mode description)))
-  (output-port/flush-output console-i/o-port))
+  (output-port/flush-output (console-i/o-port)))
 
 (define (console-modeline-event! screen window type)
   screen window type
   unspecific)
 
 (define (console-wrap-update! screen thunk)
-  screen
   (let ((finished? (thunk)))
     (window-direct-output-cursor! (screen-cursor-window screen))
-    (output-port/flush-output console-i/o-port)
+    (output-port/flush-output (console-i/o-port))
     finished?))
 
 (define (console-discretionary-flush screen)
-  (let ((n (output-port/buffered-bytes console-i/o-port)))
+  (let ((n (output-port/buffered-bytes (console-i/o-port))))
     (if (fix:< 20 n)
 	(begin
-	  (output-port/flush-output console-i/o-port)
+	  (output-port/flush-output (console-i/o-port))
 	  (let ((baud-rate (screen-baud-rate screen)))
 	    (if (fix:< baud-rate 2400)
 		(let ((msec (quotient (* n 10000) baud-rate)))
@@ -535,7 +582,7 @@ USA.
 
 (define (console-flush! screen)
   screen
-  (output-port/flush-output console-i/o-port))
+  (output-port/flush-output (console-i/o-port)))
 
 (define (console-write-cursor! screen x y)
   (move-cursor screen x y))
@@ -549,7 +596,7 @@ USA.
 	(exit-insert-mode screen)
 	(move-cursor screen x y)
 	(highlight-if-desired screen highlight)
-	(output-port/write-char console-i/o-port char)
+	(output-port/write-char (console-i/o-port) char)
 	(record-cursor-after-output screen (fix:1+ x)))))
 
 (define (console-write-substring! screen x y string start end highlight)
@@ -566,7 +613,7 @@ USA.
 				 (screen-x-size screen))))
 		   (fix:-1+ end)
 		   end)))
-	  (output-port/write-substring console-i/o-port string start end)
+	  (output-port/write-substring (console-i/o-port) string start end)
 	  (record-cursor-after-output screen (fix:+ x (fix:- end start)))))))
 
 (define (console-clear-line! screen x y first-unused-x)
@@ -614,7 +661,7 @@ USA.
 		 (and (fix:< (insert-lines-cost screen yl yu amount) draw-cost)
 		      (begin
 			(insert-lines screen yl yu amount)
-			'CLEARED))
+			'cleared))
 		 (and (fix:<
 		       (fix:+ (delete-lines-cost screen yu* y-size amount)
 			      (insert-lines-cost screen yl y-size amount))
@@ -622,7 +669,7 @@ USA.
 		      (begin
 			(delete-lines screen yu* y-size amount)
 			(insert-lines screen yl y-size amount)
-			'CLEARED))))))))
+			'cleared))))))))
 
 (define (console-scroll-lines-up! screen xl xu yl yu amount)
   (let ((description (screen-description screen)))
@@ -636,7 +683,7 @@ USA.
 	       (and (fix:< (delete-lines-cost screen yl yu amount) draw-cost)
 		    (begin
 		      (delete-lines screen yl yu amount)
-		      'CLEARED))
+		      'cleared))
 	       (let ((yu* (fix:- yu amount)))
 		 (and (fix:<
 		       (fix:+ (delete-lines-cost screen yl y-size amount)
@@ -645,7 +692,7 @@ USA.
 		      (begin
 			(delete-lines screen yl y-size amount)
 			(insert-lines screen yu* y-size amount)
-			'CLEARED))))))))
+			'cleared))))))))
 
 (define (scroll-draw-cost screen yl yu)
   (do ((yl yl (fix:+ yl 1))
@@ -698,7 +745,7 @@ USA.
 		       first-unused-x)))
 	      (do ((x (screen-cursor-x screen) (fix:1+ x)))
 		  ((fix:= x first-unused-x))
-		(output-port/write-char console-i/o-port #\space))
+		(output-port/write-char (console-i/o-port) #\space))
 	      (record-cursor-after-output screen first-unused-x)))))))
 
 (define (clear-multi-char screen n)
@@ -723,7 +770,7 @@ USA.
 			   x-end))))
 		(do ((x cursor-x (fix:1+ x)))
 		    ((fix:= x x-end))
-		  (output-port/write-char console-i/o-port #\space))
+		  (output-port/write-char (console-i/o-port) #\space))
 		(record-cursor-after-output screen x-end))))))))
 
 (define (insert-lines screen yl yu n)
@@ -1044,7 +1091,7 @@ USA.
   (output-n screen command 1))
 
 (define-integrable (output-n screen command n-lines)
-  (output-port/write-string console-i/o-port
+  (output-port/write-string (console-i/o-port)
 			    (pad-string screen command n-lines)))
 
 (define (maybe-output screen command)
@@ -1152,7 +1199,7 @@ compute this as INSERT-LINE-COST[line]+INSERT-LINE-NEXT-COST[line], we
 add INSERT-LINE-NEXT-COST into INSERT-LINE-COST.  This is reasonable
 because of the particular algorithm used.
 
-Deletion is essentially the same as insertion. 
+Deletion is essentially the same as insertion.
 
 Note that the multiply factors are in tenths of characters.  |#
 
@@ -1176,16 +1223,31 @@ Note that the multiply factors are in tenths of characters.  |#
 	 (state (screen-state screen)))
     (if (not (terminal-state? state))
 	(editor-error "Not a terminal screen")
-	(let ((port console-i/o-port)
+	(let ((port (console-i/o-port))
 	      (desc (terminal-state/description state)))
 	  (let ((x-size (output-port/x-size port))
 		(y-size (output-port/y-size port)))
 	    (if (or (not (= x-size (screen-x-size screen)))
 		    (not (= y-size (screen-y-size screen))))
 		(begin
-		  (without-interrupts
-		   (lambda ()
-		     (set-tn-x-size! desc x-size)
-		     (set-tn-y-size! desc y-size)
-		     (set-screen-size! screen x-size y-size)))
-		  (update-screen! screen #t))))))))
\ No newline at end of file
+		  (update-terminal-size! screen state desc x-size y-size)
+		  (update-screen! screen #t))))))))
+
+(define (update-terminal-size! screen state desc x-size y-size)
+  (receive (insert-line-cost
+	    insert-line-next-cost
+	    delete-line-cost
+	    delete-line-next-cost
+	    scroll-region-cost)
+      (let ((baud-rate (terminal-state/baud-rate state)))
+	(compute-scrolling-costs desc baud-rate x-size y-size))
+    (without-interrupts
+     (lambda ()
+       (set-tn-x-size! desc x-size)
+       (set-tn-y-size! desc y-size)
+       (set-terminal-state/insert-line-cost! state insert-line-cost)
+       (set-terminal-state/insert-line-next-cost! state insert-line-next-cost)
+       (set-terminal-state/delete-line-cost! state delete-line-cost)
+       (set-terminal-state/delete-line-next-cost! state delete-line-next-cost)
+       (set-terminal-state/scroll-region-cost! state scroll-region-cost)
+       (set-screen-size! screen x-size y-size)))))
\ No newline at end of file
