@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -68,16 +68,16 @@ typedef SCHEME_OBJECT word_t;	/* convenience abbreviation */
 static word_t word_registers [N_WORD_REGISTERS];
 static double float_registers [N_FLOAT_REGISTERS];
 
-#define SBYTE (sizeof (byte_t))
+#define SBYTE (sizeof (uint8_t))
 #define SWORD (sizeof (word_t))
 #define SFLOAT (sizeof (double))
 
 #define PC program_counter
 #define NEXT_BYTE (*program_counter++)
 
-typedef byte_t wreg_t;
-typedef byte_t freg_t;
-typedef byte_t tc_t;
+typedef uint8_t wreg_t;
+typedef uint8_t freg_t;
+typedef uint8_t tc_t;
 
 #define WREG_REF(wr) (word_registers [(wr)])
 #define FREG_REF(fr) (float_registers [(fr)])
@@ -85,7 +85,7 @@ typedef byte_t tc_t;
 #define WREG_SET(wr, w) ((word_registers [(wr)]) = (w))
 #define FREG_SET(fr, f) ((float_registers [(fr)]) = (f))
 
-#define BYTE_ADDR(a) ((byte_t *) a)
+#define BYTE_ADDR(a) ((uint8_t *) a)
 #define WORD_ADDR(a) ((word_t *) a)
 #define FLOAT_ADDR(a) ((double *) a)
 
@@ -94,10 +94,10 @@ typedef byte_t tc_t;
 #define FLOAT_REF(a) (* (FLOAT_ADDR (a)))
 
 
-typedef byte_t * inst_defn_t (byte_t *);
+typedef uint8_t * inst_defn_t (uint8_t *);
 static inst_defn_t * inst_defns [256];
 
-#define DEFINE_INST(name) static byte_t * insn_##name (byte_t * PC)
+#define DEFINE_INST(name) static uint8_t * insn_##name (uint8_t * PC)
 #define NEXT_PC return (PC)
 #define OFFSET_PC(o) do							\
 {									\
@@ -126,38 +126,38 @@ static long svm1_result;
 
 #define	ILL EXIT_VM (ERR_COMPILED_CODE_ERROR)
 
-typedef byte_t * address_decoder_t (byte_t *, word_t *);
+typedef uint8_t * address_decoder_t (uint8_t *, word_t *);
 static address_decoder_t * address_decoders [256];
 
 #define ADDRESS_VALUE(name) name
 
 #define DEFINE_ADDRESS_DECODER(name)					\
-  static byte_t * decode_addr_##name (byte_t * PC,			\
-				      word_t * address)
+  static uint8_t * decode_addr_##name (uint8_t * PC,			\
+                                       word_t * address)
 #define ADDRESS_DECODED(addr) (*address) = (addr); return (PC)
 #define DECODE_ADDRESS(name)						\
   word_t name;								\
   {									\
-    byte_t * new_pc = decode_address (PC, &name);			\
+    uint8_t * new_pc = decode_address (PC, &name);			\
     if (predict_false (! new_pc)) ILL;					\
     PC = new_pc;							\
   }
-static byte_t * decode_address (byte_t *, word_t *);
+static uint8_t * decode_address (uint8_t *, word_t *);
 
-typedef byte_t * trap_0_t (byte_t *);
-typedef byte_t * trap_1_t (byte_t *, wreg_t);
-typedef byte_t * trap_2_t (byte_t *, wreg_t, wreg_t);
-typedef byte_t * trap_3_t (byte_t *, wreg_t, wreg_t, wreg_t);
+typedef uint8_t * trap_0_t (uint8_t *);
+typedef uint8_t * trap_1_t (uint8_t *, wreg_t);
+typedef uint8_t * trap_2_t (uint8_t *, wreg_t, wreg_t);
+typedef uint8_t * trap_3_t (uint8_t *, wreg_t, wreg_t, wreg_t);
 
 static trap_0_t * traps_0 [256];
 static trap_1_t * traps_1 [256];
 static trap_2_t * traps_2 [256];
 static trap_3_t * traps_3 [256];
 
-#define DECODE_TRAP_0(name) byte_t name = NEXT_BYTE
-#define DECODE_TRAP_1(name) byte_t name = NEXT_BYTE
-#define DECODE_TRAP_2(name) byte_t name = NEXT_BYTE
-#define DECODE_TRAP_3(name) byte_t name = NEXT_BYTE
+#define DECODE_TRAP_0(name) uint8_t name = NEXT_BYTE
+#define DECODE_TRAP_1(name) uint8_t name = NEXT_BYTE
+#define DECODE_TRAP_2(name) uint8_t name = NEXT_BYTE
+#define DECODE_TRAP_3(name) uint8_t name = NEXT_BYTE
 
 static void initialize_decoder_tables (void);
 
@@ -212,22 +212,45 @@ initialize_svm1 (void)
   SET_VAL ((SCHEME_OBJECT) (WREG_REF (SVM1_REG_VALUE)));		\
 } while (0)
 
+/* Only true when the virtual machine is running, e.g. while Free is
+   cached in word_registers[].  In a signal handler, the virtual
+   instruction that trapped is at instruction_pointer. */
+static bool running_p = false;
+static insn_t * instruction_pointer;
+
 long
 C_to_interface (void * address)
 {
-  insn_t * PC = (insn_t *)address;
+  instruction_pointer = (insn_t *)address;
   IMPORT_REGS ();
-  while (predict_true (PC))
+  running_p = true;
+  while (predict_true (instruction_pointer))
     {
-      byte_t opcode = *PC++;
-      PC = (* (inst_defns[opcode])) (PC);
+      uint8_t opcode = *instruction_pointer;
+      instruction_pointer = (* (inst_defns[opcode])) (instruction_pointer+1);
     }
   EXPORT_REGS ();
+  running_p = false;
   return (svm1_result);
 }
 
-static byte_t *
-illegal_instruction (byte_t * PC)
+/* For signal/trap handlers. */
+unsigned long
+svm_export_instruction_pointer (unsigned long pc)
+{
+  if (running_p)
+    {
+      EXPORT_REGS ();
+      return ((unsigned long)instruction_pointer);
+    }
+  else
+    {
+      return (pc);
+    }
+}
+
+static uint8_t *
+illegal_instruction (uint8_t * PC)
 {
   ILL;
 }
@@ -261,12 +284,12 @@ to_signed (word_t n)
 
 /* Primitive decoders */
 
-#define DECODE_ARG(name,func) do					\
-  {									\
-    byte_t * new_pc = func (PC, &name);			\
-    if (predict_false (! new_pc)) ILL;					\
-    PC = new_pc;						\
-  } while (0);
+#define DECODE_ARG(name, func) do                                       \
+{                                                                       \
+  uint8_t * new_pc = func (PC, &name);                                  \
+  if (predict_false (! new_pc)) ILL;                                    \
+  PC = new_pc;                                                          \
+} while (0);
 
 #define DECODE_WORD_REGISTER(name)  wreg_t name; DECODE_ARG(name,decode_wreg)
 #define DECODE_FLOAT_REGISTER(name) freg_t name; DECODE_ARG(name,decode_freg)
@@ -279,45 +302,45 @@ to_signed (word_t n)
 #define DECODE_SIGNED_32(name)        long name; DECODE_ARG(name,decode_signed_32)
 #define DECODE_FLOAT(name)          double name; DECODE_ARG(name,decode_float)
 
-static inline byte_t *
-decode_wreg (byte_t * PC, wreg_t *wreg)
+static inline uint8_t *
+decode_wreg (uint8_t * PC, wreg_t *wreg)
 {
-  byte_t b = NEXT_BYTE;
+  uint8_t b = NEXT_BYTE;
   if (predict_false (! (WORD_REGISTER_P (b))))
     return (NULL);
   *wreg = b;
   return (PC);
 }
 
-static inline byte_t *
-decode_freg (byte_t * PC, freg_t *freg)
+static inline uint8_t *
+decode_freg (uint8_t * PC, freg_t *freg)
 {
-  byte_t b = NEXT_BYTE;
+  uint8_t b = NEXT_BYTE;
   if (predict_false (! (FLOAT_REGISTER_P (b))))
     return (NULL);
   *freg = b;
   return (PC);
 }
 
-static inline byte_t *
-decode_type_word (byte_t * PC, tc_t *tc)
+static inline uint8_t *
+decode_type_word (uint8_t * PC, tc_t *tc)
 {
-  byte_t b = NEXT_BYTE;
+  uint8_t b = NEXT_BYTE;
   if (predict_false (! (b <= N_TYPE_CODES)))
     return (NULL);
   *tc = b;
   return (PC);
 }
 
-static inline byte_t *
-decode_unsigned_8 (byte_t * PC, word_t *word)
+static inline uint8_t *
+decode_unsigned_8 (uint8_t * PC, word_t *word)
 {
   *word = (NEXT_BYTE);
   return (PC);
 }
 
-static inline byte_t *
-decode_unsigned_16 (byte_t * PC, word_t *word)
+static inline uint8_t *
+decode_unsigned_16 (uint8_t * PC, word_t *word)
 {
   word_t b0 = NEXT_BYTE;
   word_t b1 = NEXT_BYTE;
@@ -325,8 +348,8 @@ decode_unsigned_16 (byte_t * PC, word_t *word)
   return (PC);
 }
 
-static inline byte_t *
-decode_unsigned_32 (byte_t * PC, word_t *word)
+static inline uint8_t *
+decode_unsigned_32 (uint8_t * PC, word_t *word)
 {
   word_t b0 = NEXT_BYTE;
   word_t b1 = NEXT_BYTE;
@@ -336,8 +359,8 @@ decode_unsigned_32 (byte_t * PC, word_t *word)
   return (PC);
 }
 
-static inline byte_t *
-decode_unsigned_64 (byte_t * PC, uint64_t *big)
+static inline uint8_t *
+decode_unsigned_64 (uint8_t * PC, uint64_t *big)
 {
   uint64_t b0, b1, b2, b3, b4, b5, b6, b7;
   b0 = NEXT_BYTE; b1 = NEXT_BYTE; b2 = NEXT_BYTE; b3 = NEXT_BYTE;
@@ -347,16 +370,16 @@ decode_unsigned_64 (byte_t * PC, uint64_t *big)
   return (PC);
 }
 
-static inline byte_t *
-decode_signed_8 (byte_t * PC, long *lng)
+static inline uint8_t *
+decode_signed_8 (uint8_t * PC, long *lng)
 {
   long b = NEXT_BYTE;
   *lng = ((b < 0x80) ? b : (b - 0x100));
   return (PC);
 }
 
-static inline byte_t *
-decode_signed_16 (byte_t * PC, long *lng)
+static inline uint8_t *
+decode_signed_16 (uint8_t * PC, long *lng)
 {
   unsigned long n;
   PC = decode_unsigned_16 (PC, &n);
@@ -365,8 +388,8 @@ decode_signed_16 (byte_t * PC, long *lng)
   return (PC);
 }
 
-static inline byte_t *
-decode_signed_32 (byte_t * PC, long *lng)
+static inline uint8_t *
+decode_signed_32 (uint8_t * PC, long *lng)
 {
   word_t n;
   PC = (decode_unsigned_32 (PC, &n));
@@ -390,8 +413,8 @@ decode_signed_32 (byte_t * PC, long *lng)
   return (PC);
 }
 
-static inline byte_t *
-decode_signed_64 (byte_t * PC, int64_t * big)
+static inline uint8_t *
+decode_signed_64 (uint8_t * PC, int64_t * big)
 {
   uint64_t n;
   PC = (decode_unsigned_64 (PC, &n));
@@ -411,8 +434,8 @@ decode_signed_64 (byte_t * PC, int64_t * big)
   }
 }
 
-static inline byte_t *
-decode_float (byte_t * PC, double * dbl)
+static inline uint8_t *
+decode_float (uint8_t * PC, double * dbl)
 {
   int64_t significand;
   long exponent;
@@ -512,6 +535,9 @@ DEFINE_INST (load_immediate_fr_flt)
 
 #define X_OBJECT_ADDRESS(o) ((word_t) (OBJECT_ADDRESS (o)))
 
+#define X_FIXNUM_TO_LONG(o)						\
+  ((((long) (o)) << TYPE_CODE_LENGTH) >> TYPE_CODE_LENGTH)
+
 DEFINE_INST (load_non_pointer_tc_s8)
 {
   DECODE_SVM1_INST_LOAD_NON_POINTER_TC_S8 (target, type, datum);
@@ -666,17 +692,26 @@ DEFINE_INST (ijump_u32)
 static inline void
 push_object (SCHEME_OBJECT object)
 {
-  stack_pointer = ((SCHEME_OBJECT *) (WREG_REF (SVM1_REG_STACK_POINTER)));
-  STACK_PUSH (object);
-  WREG_SET (SVM1_REG_STACK_POINTER, ((SCHEME_OBJECT) stack_pointer));
+  SCHEME_OBJECT * sp = ((SCHEME_OBJECT *) (WREG_REF (SVM1_REG_STACK_POINTER)));
+  (STACK_LOCATIVE_PUSH (sp)) = object;
+  WREG_SET (SVM1_REG_STACK_POINTER, ((word_t) sp));
 }
 
 static inline void
-push_entry (byte_t * PC)
+push_entry (uint8_t * PC)
 {
   push_object (MAKE_CC_ENTRY (PC + CC_ENTRY_HEADER_SIZE));
 }
 
+static inline SCHEME_OBJECT
+pop_object (void)
+{
+  SCHEME_OBJECT * sp = ((SCHEME_OBJECT *) (WREG_REF (SVM1_REG_STACK_POINTER)));
+  SCHEME_OBJECT object = (STACK_LOCATIVE_POP (sp));
+  WREG_SET (SVM1_REG_STACK_POINTER, ((word_t) sp));
+  return (object);
+}
+
 DEFINE_INST (icall_u8)
 {
   DECODE_SVM1_INST_ICALL_U8 (offset);
@@ -804,8 +839,8 @@ DEFINE_INST (trap_trap_0)
   return ((* (traps_0[code])) (PC));
 }
 
-static byte_t *
-illegal_trap_0 (byte_t * PC)
+static uint8_t *
+illegal_trap_0 (uint8_t * PC)
 {
   ILL;
 }
@@ -816,8 +851,8 @@ DEFINE_INST (trap_trap_1_wr)
   return ((* (traps_1[code])) (PC, r1));
 }
 
-static byte_t *
-illegal_trap_1 (byte_t * PC, wreg_t r1)
+static uint8_t *
+illegal_trap_1 (uint8_t * PC, wreg_t r1)
 {
   ILL;
 }
@@ -828,8 +863,8 @@ DEFINE_INST (trap_trap_2_wr)
   return ((* (traps_2[code])) (PC, r1, r2));
 }
 
-static byte_t *
-illegal_trap_2 (byte_t * PC, wreg_t r1, wreg_t r2)
+static uint8_t *
+illegal_trap_2 (uint8_t * PC, wreg_t r1, wreg_t r2)
 {
   ILL;
 }
@@ -840,18 +875,20 @@ DEFINE_INST (trap_trap_3_wr)
   return ((* (traps_3[code])) (PC, r1, r2, r3));
 }
 
-static byte_t *
-illegal_trap_3 (byte_t * PC, wreg_t r1, wreg_t r2, wreg_t r3)
+static uint8_t *
+illegal_trap_3 (uint8_t * PC, wreg_t r1, wreg_t r2, wreg_t r3)
 {
   ILL;
 }
 
 #define TRAP_PREFIX(result)						\
   utility_result_t result;						\
-  EXPORT_REGS ()
+  EXPORT_REGS ();							\
+  running_p = false
 
 #define TRAP_SUFFIX(result)						\
   IMPORT_REGS ();							\
+  running_p = true;							\
   if ((result).scheme_p)						\
     {									\
       NEW_PC ((result).arg.new_pc);					\
@@ -860,8 +897,8 @@ illegal_trap_3 (byte_t * PC, wreg_t r1, wreg_t r2, wreg_t r3)
     EXIT_VM ((result).arg.interpreter_code)
 
 #define DEFINE_TRAP_0(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC)							\
+static uint8_t *							\
+trap_##nl (uint8_t * PC)						\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -873,8 +910,8 @@ trap_##nl (byte_t * PC)							\
 }
 
 #define DEFINE_TRAP_1(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC, wreg_t source1)					\
+static uint8_t *							\
+trap_##nl (uint8_t * PC, wreg_t source1)				\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -886,8 +923,8 @@ trap_##nl (byte_t * PC, wreg_t source1)					\
 }
 
 #define DEFINE_TRAP_2(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2)			\
+static uint8_t *							\
+trap_##nl (uint8_t * PC, wreg_t source1, wreg_t source2)		\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -899,8 +936,8 @@ trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2)			\
 }
 
 #define DEFINE_TRAP_3(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2, wreg_t source3)	\
+static uint8_t *							\
+trap_##nl (uint8_t * PC, wreg_t source1, wreg_t source2, wreg_t source3) \
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -912,8 +949,8 @@ trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2, wreg_t source3)	\
 }
 
 #define DEFINE_TRAP_R0(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC)							\
+static uint8_t *							\
+trap_##nl (uint8_t * PC)						\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -925,8 +962,8 @@ trap_##nl (byte_t * PC)							\
 }
 
 #define DEFINE_TRAP_R1(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC, wreg_t source1)					\
+static uint8_t *							\
+trap_##nl (uint8_t * PC, wreg_t source1)				\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -938,8 +975,8 @@ trap_##nl (byte_t * PC, wreg_t source1)					\
 }
 
 #define DEFINE_TRAP_R2(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2)			\
+static uint8_t *							\
+trap_##nl (uint8_t * PC, wreg_t source1, wreg_t source2)		\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -951,8 +988,8 @@ trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2)			\
 }
 
 #define DEFINE_TRAP_R3(nl, util_name)					\
-byte_t *								\
-trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2, wreg_t source3)	\
+static uint8_t *							\
+trap_##nl (uint8_t * PC, wreg_t source1, wreg_t source2, wreg_t source3) \
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name ((&result),					\
@@ -964,8 +1001,8 @@ trap_##nl (byte_t * PC, wreg_t source1, wreg_t source2, wreg_t source3)	\
 }
 
 #define DEFINE_TRAMPOLINE(nl, util_name)				\
-byte_t *								\
-trap_##nl (byte_t * PC)							\
+static uint8_t *							\
+trap_##nl (uint8_t * PC)						\
 {									\
   TRAP_PREFIX (result);							\
   comutil_##util_name							\
@@ -1056,6 +1093,11 @@ DEFINE_INTERRUPT_TEST (dynamic_link,
 		       (PC - 1),
 		       (MAKE_CC_STACK_ENV (WREG_REF (SVM1_REG_DYNAMIC_LINK))))
 
+DEFINE_INST (pop_return)
+{
+  return (BYTE_ADDR (OBJECT_ADDRESS (pop_object ())));
+}
+
 DEFINE_INST (enter_closure)
 {
   DECODE_SVM1_INST_ENTER_CLOSURE (index);
@@ -1068,8 +1110,8 @@ DEFINE_INST (enter_closure)
     }
 
   {
-    byte_t * block = (PC - (CLOSURE_ENTRY_START
-			    + ((index + 1) * CLOSURE_ENTRY_SIZE)));
+    uint8_t * block = (PC - (CLOSURE_ENTRY_START
+                             + ((index + 1) * CLOSURE_ENTRY_SIZE)));
     unsigned int count = (read_u16 (block));
     SCHEME_OBJECT * targets
       = (skip_compiled_closure_padding
@@ -1163,7 +1205,7 @@ DEFINE_UNARY_WR (not, NOT, ~)
 DEFINE_UNARY_WR (object_type, OBJECT_TYPE, OBJECT_TYPE)
 DEFINE_UNARY_WR (object_datum, OBJECT_DATUM, OBJECT_DATUM)
 DEFINE_UNARY_WR (object_address, OBJECT_ADDRESS, X_OBJECT_ADDRESS)
-DEFINE_UNARY_WR (fixnum_to_integer, FIXNUM_TO_INTEGER, FIXNUM_TO_LONG)
+DEFINE_UNARY_WR (fixnum_to_integer, FIXNUM_TO_INTEGER, X_FIXNUM_TO_LONG)
 DEFINE_UNARY_WR (integer_to_fixnum, INTEGER_TO_FIXNUM, LONG_TO_FIXNUM)
 DEFINE_UNARY_WR (flonum_length, FLONUM_LENGTH, FLOATING_VECTOR_LENGTH)
 
@@ -1264,15 +1306,15 @@ DEFINE_BINARY_FR (atan2, ATAN2, atan2)
 
 /* Address decoders */
 
-static inline byte_t *
-decode_address (byte_t * PC, word_t * address)
+static inline uint8_t *
+decode_address (uint8_t * PC, word_t * address)
 {
   address_decoder_t * decoder = address_decoders[NEXT_BYTE];
   return ((*decoder) (PC, address));
 }
 
-static byte_t *
-illegal_address (byte_t * PC, word_t * address)
+static uint8_t *
+illegal_address (uint8_t * PC, word_t * address)
 {
   (void) PC;			/* ignore */
   (void) address;		/* ignore */
@@ -1510,7 +1552,7 @@ DEFINE_ADDRESS_DECODER (postinc_f)
 }
 
 static inline word_t
-pcr_value (byte_t * PC, word_t offset)
+pcr_value (uint8_t * PC, word_t offset)
 {
   return (((word_t) PC) + offset);
 }
